@@ -5,6 +5,7 @@ import math
 from os import path
 from copy import deepcopy
 import json
+import re
 
 from ._schema_common import (
     PRIMITIVES,
@@ -19,6 +20,8 @@ from ._schema_common import (
     rabin_fingerprint,
 )
 
+SYMBOL_REGEX = re.compile(r"[A-Za-z_][A-Za-z0-9_]*")
+
 
 def extract_record_type(schema):
     if isinstance(schema, dict):
@@ -220,11 +223,19 @@ def parse_schema(
     if named_schemas is None:
         named_schemas = {}
 
+    if isinstance(schema, dict) and "__fastavro_parsed" in schema:
+        if "__named_schemas" in schema:
+            for key, value in schema["__named_schemas"].items():
+                named_schemas[key] = value
+        else:
+            # Some old schemas might only have __fastavro_parsed and not
+            # __named_schemas since that came later. For these schemas, we need
+            # to re-parse the schema to handle named types
+            return _parse_schema(schema, "", expand, _write_hint, set(), named_schemas)
+
     if _force or expand:
         return _parse_schema(schema, "", expand, _write_hint, set(), named_schemas)
     elif isinstance(schema, dict) and "__fastavro_parsed" in schema:
-        for key, value in schema["__named_schemas"].items():
-            named_schemas[key] = value
         return schema
     elif isinstance(schema, list):
         # If we are given a list we should make sure that the immediate sub
@@ -342,6 +353,8 @@ def _parse_schema(schema, namespace, expand, _write_hint, names, named_schemas):
                 raise SchemaParseException(f"redefined named type: {fullname}")
             names.add(fullname)
 
+            _validate_enum_symbols(schema["symbols"])
+
             named_schemas[fullname] = parsed_schema
 
             parsed_schema["name"] = fullname
@@ -770,3 +783,13 @@ def fingerprint(parsing_canonical_form, algorithm):
 
     h = hashlib.new(algorithm, parsing_canonical_form.encode())
     return h.hexdigest()
+
+
+def _validate_enum_symbols(symbols):
+    for symbol in symbols:
+        if not isinstance(symbol, str) or not re.match(SYMBOL_REGEX, symbol):
+            raise SchemaParseException(
+                "Every symbol must match the regular expression [A-Za-z_][A-Za-z0-9_]*"
+            )
+    if len(symbols) != len(set(symbols)):
+        raise SchemaParseException("All symbols in an enum must be unique")
